package knockmonitor;
import java.util.*;
import java.net.*;
import java.io.*;
import javax.swing.*;

public class KnockThread extends Thread {
    OptionsDB optionsDB;
    JTextArea taOutput;
    boolean bStop;
    String knockingIP, MAC = "", MS = "", MD = "", Interface = "";
    Vector<String> sequence;    
    public KnockThread(OptionsDB optionsDB, JTextArea taOutput) {
        this.optionsDB = optionsDB;
        this.taOutput = taOutput;
        bStop = false;
    }    
    public void run() {
        String logFileName;
        int port;             
        String localIp = "";        
        //get local ip address
        try {
            //get all local addresses and use the first one that doesn't start with a zero
            //this gets round the case where a 'dummy' nic is set up in network connections for a webcam
            InetAddress [] locaAddresses = InetAddress.getAllByName(InetAddress.getLocalHost().getHostName());
            for(int i=0; i<locaAddresses.length && localIp.length() == 0; i++) {
                if(!locaAddresses[i].getHostAddress().startsWith("0")) {
                    localIp = locaAddresses[i].getHostAddress();
                }
            }
            taOutput.append("Local IP address = " + localIp + "\n");
        } catch (UnknownHostException e) {
            e.printStackTrace();
        }        
        taOutput.append("Starting Knock Monitoring");        
        while (!bStop) {
            synchronized (optionsDB) {
		logFileName = optionsDB.logFileName;
                port = optionsDB.port;
		if(optionsDB.optionsChanged)
		{
		    taOutput.append("\nOptions saved\n");
		    deleteLogFile(new File(logFileName), true);
		    optionsDB.optionsChanged = false;
		}                
                //sequence = (Vector<String>)optionsDB.sequence.clone();
                sequence = new Vector<String>(optionsDB.sequence);                
                if (logFileName == null || logFileName.equals("") || port == 0 || sequence.size() == 0) {
                    try {
                        optionsDB.wait();
                    } catch (Exception e) {
                        e.printStackTrace();
                        continue;
                    }
                }
                File logFile = new File(logFileName);
                knockingIP = "";
                if(lookForKnock(logFile)) {
		    deleteLogFile(logFile, false);                    
                    taOutput.append("Opening port " + port + " for " + knockingIP + "...\n");
                    //this needs to be changed to the location of your CHX-I packet filter - might just move ftlcon.exe to the working directory later
                    //FIXME: the filter set up in CHX doesn't have the dest. port - it just shows 'any'
                    //CmdExec("\"F:\\Program Files\\CHX-I\\fltcon.exe\" /ADD /DIR 0 /PY 4 /SA " + knockingIP + " /P 6 /DA " + localIp + " /DM 255.255.255.255 /DP " + port);
		    //CmdExec(optionsDB.chxPfConLocation + " /ADD /T 1800 /DIR 0 /PY 4 /SA " + knockingIP + " /P 6 /DA " + localIp + " /DM 255.255.255.255 /DP " + port);
		    //CmdExec(optionsDB.chxPfConLocation + " /ADD /T 1800 /PY 4 /DIR 0 /SA " + knockingIP + " /SM 255.255.255.255 /DA " + localIp + " /DM 255.255.255.255 /P 6 /DP " + port);
		    //CmdExec(optionsDB.chxPfConLocation + " /ADD /T 1800 /PY 4 /DIR 0 /MAC " + MAC + " /MS " + MS + " /MD " + MD + " /SA " + knockingIP + " /SM 255.255.255.255 /DA " + localIp + " /DM 255.255.255.255");
		    CmdExec(optionsDB.chxPfConLocation + " /ADD /T 1800 /PY 4 /DIR 0 /P 6 /SA " + knockingIP + " /SM 255.255.255.255 /DA " + localIp + " /DM 255.255.255.255 /DP " + port + " /A 0");
		    //taOutput.append("MAC = " + MAC);
                }
                try {
                    Thread.sleep(1000);
                } catch (Exception ex) {
                    ex.printStackTrace();
                }
            }
        }
    }
    public void deleteLogFile(File logFile, boolean quiet)
    {
	if(!quiet)
	{
	    taOutput.append("Deleting log file...");
	    taOutput.append((logFile.delete() ? "OK" : "FAILED"));	    
	}
        taOutput.append("\n");
    }
    public boolean lookForKnock(File aFile) {
        BufferedReader input = null;
        String afterFromPort, port, ip, result;
        String [] lineValues = null;        
        boolean knockSequenceFound = false;
        int sequenceIndex = 0, consecutiveMatches = 0;
        try {
            input = new BufferedReader( new FileReader(aFile) );
            result = "";
            String line = null;
            while((line = input.readLine()) != null) //for each line
            {
                //parse this line and compare incoming port with knock sequence
                //the current logic is that if the ports are found in the right order, then the knock sequence has been found (irrespective of the ip they came from)
                lineValues = line.split(",");
                //lineValues[0] =  Date
                //lineValues[1] =  Time
                //lineValues[2] =  Direction
                //lineValues[3] =  Interface
                //lineValues[4] =  EthType
                //lineValues[5] =  Source MAC
                //lineValues[6] =  Destination MAC
                //lineValues[7] =  Protocol
                //lineValues[8] =  Flags
                //lineValues[9] =  Source IP
                //lineValues[10] =  Source Port
                //lineValues[11] =  Destination IP
                //lineValues[12] =  Destination Port
                //lineValues[13] =  Packet size
                //lineValues[14] =  Reason
//              taOutput.append("\nlineValues.length = "+lineValues.length);
//                for(int i=0; i<lineValues.length; i++)
//                    taOutput.append("\nlinesValue["+i+"] = "+lineValues[i]);
                //ip = line.substring(line.indexOf("from IP:") + 8, line.indexOf(",",line.indexOf("from IP:")));
                ip = lineValues[9].trim();                
                //afterFromPort = line.substring(line.indexOf("Port:")+5);
                //port = afterFromPort.substring(afterFromPort.indexOf("Port:") + 5, afterFromPort.indexOf(",",afterFromPort.indexOf("Port:")));
                port = lineValues[12].trim();
                if(sequenceIndex < sequence.size() && port.compareTo(sequence.get(sequenceIndex)) == 0) {
                    result += "\nFrom " + ip + " to port: " + port;
                    sequenceIndex++;
                    consecutiveMatches++;
                    knockingIP = ip;
		    MAC = lineValues[6].trim().replaceAll(" ", "-");
		    MD = MAC;
		    MS = lineValues[5].trim().replaceAll(" ", "-");
		    Interface = lineValues[3].trim();
                }
            }
//            if(result.length() > 0)
//            {
//                result += "\nconsecutiveMatches = " + consecutiveMatches;
//                taOutput.append(result);
//            }
            if(consecutiveMatches == sequence.size() && sequence.size() != 0) {
                taOutput.append("\n\nKnock Sequence Found\n");
                taOutput.append(result + "\n");    
                knockSequenceFound = true;
            } else {
                //System.out.print(".");
            }
        } catch (FileNotFoundException ex) {
            //System.out.print("!");
            //ex.printStackTrace();
        } catch (IOException ex){ ex.printStackTrace(); } finally {
            try {
                if (input!= null) {
                    //flush and close both "input" and its underlying FileReader
                    input.close();
                }
            } catch (IOException ex) { ex.printStackTrace(); }
        }
        return knockSequenceFound;
    }
    public void CmdExec(String cmdline) { //executes 'cmdline' in the windows shell
        try {
            String line;
            taOutput.append(cmdline);
            taOutput.append("\n");
            Process p = Runtime.getRuntime().exec(cmdline);
            BufferedReader input = new BufferedReader(new InputStreamReader(p.getInputStream()));
            while ((line = input.readLine()) != null) {
                taOutput.append(line);
            }
            input.close();
        } catch (Exception err) {
            err.printStackTrace();
        }
    }
}
